{# By default, this just includes the django template defined from the view/form. Override this template to change that #}

{% if html_template_name %}
    {% include html_template_name %}
{% else %}
    <pre>{% include template_name %}</pre>
{% endif %}